#!/usr/bin/python3
# -*- coding: utf-8 -*-

from . import createLogger, _CONFIG_DIR
from pathlib import Path

DEFAULT_REPO_DIR = "/src"

logger = createLogger("gitrepo.log", stream=True)
config = {
    "repo_dir": DEFAULT_REPO_DIR,
    "recommended": False,
    "user": "git",
    "group": "git",
}
config_file = _CONFIG_DIR / "gitrepo.yaml"

import yaml
def readConfig():
    global config
    if not config_file.exists():
        with config_file.open("w") as f:
            yaml.dump(config, f)
        return
    
    with config_file.open() as f:
        config = yaml.load(f.read())
    
    if "repo_dir" not in config:
        config["repo_dir"] = DEFAULT_REPO_DIR
    
    if "recommended" not in config:
        config["recommended"] = False



import argparse 
def parseArgs():
    readConfig()
    parser = argparse.ArgumentParser(description="Create or remove git repository \
        under direcotry: {}. Currently only works for Linux. \
        Most operations are only tested under Linux".format(config["repo_dir"]))

    parser.add_argument('repo', type=str, help="Specify the repository name, for example: \
        Test.git, if .git suffix is not typed, this script will add it automatically")

    action = parser.add_mutually_exclusive_group()
    action.add_argument("-i", "--init", 
        help="Default action, Init a new git repository under {}.".format(config["repo_dir"]),
        action="store_true")

    action.add_argument("-d", "--delete", 
        help="Remove a existing repository under {}.".format(config["repo_dir"]),
        action="store_true")

    parser.add_argument("-s", "--src-dir", help="Specify the src folder for git repository, for example, the default path is `/src`.\
        Do NOT put single-slash `/` as the src folder, it will make unestimated error.")
    parser.add_argument("-u", "--user", help="Specify the git repository owner, for example, the default owner is `git`")
    parser.add_argument("-g", "--group", help="Specify the git repository owner group, for example, the default owner group is `git`")

    args = parser.parse_args()
    if args.src_dir is not None and args.src_dir != "/":
        config["src"] = args.src_dir

    config["action"] = "init"
    if args.delete:
        config["action"] = "delete"

    repo = args.repo
    if repo[-4:] != ".git":
        repo = "{}.git".format(repo)

    config["repo"] = repo

    return parser

import os, shutil
def createRepo(repo: Path):
    if repo.exists():
        logger.warning("Repository: {} exists, if you want to recreate a new repository, please delete it first.\
            Now change the owner of that repository".format(repo))
        try:
            shutil.chown(repo, config["user"], config["group"])
        except Exception as e:
            logger.critical("You may need to run this script under super user privilege.")
            logger.critical(e)
        logger.info("Repository owner changed.")
        return

    logger.info("Creating an repository: {}".format(repo))
    repo.mkdir(parents=True)
    import subprocess
    try:
        subprocess.run(["sudo", "-s", "git", "init", "--bare", "--shared", str(repo.absolute())])
        shutil.chown(repo, config["user"], config["group"])
    except Exception as e:
        logger.critical("Git may not be installed, please install git first, \
            or you are not running this command under super user privilege. You may need to run this script under super user privilege.\
            \n{}".format(fnf))
        logger.critical(e)

    logger.info("Repository created.")

def deleteRepo(repo: Path):
    if not repo.exists():
        logger.warning("Repository: {} not exists".format(repo))
        return
    logger.info("Deleteing an existing repository: {}".format(repo))
    try:
        # repo.rmdir()
        shutil.rmtree(str(repo))
    except Exception as e:
        logger.warning(e)
    logger.info("Repository deleted.")


def main():
    # if os.pl
    parser = parseArgs()
    
    repo = Path(config["repo_dir"]) / config["repo"]
    try:
        if config["action"] == "delete":
            deleteRepo(repo)
        else:
            createRepo(repo)
    except:
            # for fisrt using this script
        if config["recommended"] is False:
            parser.print_help()
            with config_file.open("w") as f:
                config["recommended"] = True
                yaml.dump(config, f)
        
    

if __name__ == "__main__":
    main()


# -*- coding: utf-8 -*-

# from distutils.core import setup
from setuptools import setup, find_packages

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

with open('requirements.txt', "r", encoding="utf-8") as f:
    requires = f.read().splitlines()

setup( 
    name = "brifuture-facilities", 
    packages = find_packages(where='.'), 
    version = "0.0.1alpha4",

    entry_points = {
        "console_scripts": [
            'bf_broadcast = myfacilities.broadcast:main',
            'bf_gitrepo   = myfacilities.gitrepo:main',
        ]
    },

    description = "BriFuture's scripts set, all scripts will be written with Python3",
    author = "BriFuture",
    author_email = "jw.brifuture@gmail.com",
    license = "GPLv3",
    url = "http://github.com/brifuture/",
    
    install_requires = requires,

    include_package_data = True,
    zip_safe=True,
    exclude_package_data = {'': ['__pycache__']},

    # download_url = "",
    keywords = [ "webserver", "shadowsocks-manager" ],
    classifiers = [ 
        "Programming Language :: Python", 
        "Programming Language :: Python :: 3" ,
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
        "Operating System :: OS Independent"
    ],

    long_description = long_description,
    long_description_content_type="text/markdown",
)


#!/usr/bin/python
# -*- coding: utf-8 -*-

from pathlib import Path
mod_suffix = ('.cpp', '.h')
import os
statement = """#pragma execution_character_set("utf-8")\n"""
statement_len = len(statement)

src_encode = "utf-8"
dst_encode = "utf-8"

def is_in_suf_list(name: Path):
    if name.suffix in mod_suffix:
        return True
    return False

# insert statement at the first line of each file whose extension is in mod_suffix


def add_state(dir: Path):
    for x in dir.iterdir():
        # print(x)
        if x.is_dir():
            print("Entering directory: ", str(x))
            add_state(x)
            print("Leaving directory: ", str(x))
        elif x.is_file() and is_in_suf_list(x):
            print(" --modified: ", x)
            with x.open(mode="r", encoding=src_encode) as f:
                raw = f.read()
                s = raw.find(statement)
                if s > -1:
                    # avoid dunplicate
                    continue
            
            with x.open(mode="w", encoding=dst_encode) as f:
                # f.seek(0)
                f.write(statement + raw)


def remove_first_line(dir: Path):
    for x in dir.iterdir():
        # print(x)

        if x.is_dir():
            print("Entering directory: ", str(x))
            remove_first_line(x)
            print("Leaving directory: ", str(x))
        elif x.is_file() and is_in_suf_list(x):
            mod = None
            with x.open(mode="r", encoding=src_encode) as f:
                raw = f.read()
                s = raw.find(statement)
                if s > -1:
                    mod = raw[s+statement_len:]
                # print(s+statement_len, mod)
                # exit(0)
            if mod is None:
                continue
            print(" --modified: ", x)
            with x.open(mode="w", encoding=dst_encode) as f:
                f.write(mod)
                # exit(0)

def change_encoding(dir: Path):
    for x in dir.iterdir():
        # print(x)

        if x.is_dir():
            print("Entering directory: ", str(x))
            change_encoding(x)
            print("Leaving directory: ", str(x))
        elif x.is_file() and is_in_suf_list(x):
            try:
                with x.open(mode="r", encoding=src_encode) as f:
                    raw = f.read()
                    # print(s+statement_len, mod)
                    # exit(0)
                print(f" --modified: {x}, src: {src_encode} => {dst_encode}")
                with x.open(mode="w", encoding=dst_encode) as f:
                    f.write(raw)
            except Exception as ex:
                print(ex, f"file: {x}")

if __name__ == "__main__":
    import sys
    import argparse
    parser = argparse.ArgumentParser()
    parser.add_argument("path", help="relative path to apply actions.")
    parser.add_argument("-a", "--add", help="add statement(s)", action="store_true")
    parser.add_argument("-r", "--remove", help="remove statement(s)", action="store_true")
    parser.add_argument("-e", "--encoding", help="change encoding of file(s)", action="store_true")
    parser.add_argument("--src_encode", help="source file encoding (default utf-8)")
    parser.add_argument("--dst_encode", help="destination file encoding (default utf-8)")
    args = parser.parse_args()
    
    if args.src_encode:
        src_encode = args.src_encode
    if args.dst_encode:
        dst_encode = args.dst_encode
    
    path = Path(args.path)

    if not path.exists():
        print("Error: Path not exists.")
        exit(1)
    if args.remove:
        print("removing")
        remove_first_line(path)
    elif args.add:
        print("inserting")
        add_state(path)
    elif args.encoding:
        print("change encoding")
        change_encoding(path)
    else:
        print(parser.print_help())

